﻿using System;
using System.Collections.Generic;
using Grasshopper.Kernel;
using Grasshopper.Kernel.Data;
using Grasshopper.Kernel.Types;
using Rhino.Geometry;

namespace Draft
{
    public class Node3D : GH_Component
    {
        Random rand;

        public Node3D()
          : base("Node", "node", "3D representation of a network node", "Draft", "Test")
        {
            rand = new Random();
        }

        protected override void RegisterInputParams(GH_Component.GH_InputParamManager pManager)
        {
            pManager.AddPointParameter("Point", "P", "Point in location of node", GH_ParamAccess.item);
            pManager[0].Optional = true;
            pManager.AddColourParameter("Colors", "C", "Colors of node", GH_ParamAccess.list);
            pManager[1].Optional = true;
        }

        protected override void RegisterOutputParams(GH_Component.GH_OutputParamManager pManager)
        {
            pManager.AddMeshParameter("Mesh", "M", "Mesh representing a node", GH_ParamAccess.item);
        }

        protected void randomlyRotateMesh(Mesh mesh)
        {
            mesh.Rotate(rand.NextDouble() * 2 * Math.PI, Vector3d.ZAxis, new Point3d());
        }

        protected override void SolveInstance(IGH_DataAccess DA)
        {
            var mesh = new Mesh();
            mesh.Vertices.Add(+1, +1, +1);
            mesh.Vertices.Add(-1, -1, +1);
            mesh.Vertices.Add(-1, +1, -1);
            mesh.Vertices.Add(+1, -1, -1);
            mesh.Faces.AddFace(0, 1, 2);
            mesh.Faces.AddFace(0, 1, 3);
            mesh.Faces.AddFace(0, 2, 3);
            mesh.Faces.AddFace(1, 2, 3);
            mesh.Normals.ComputeNormals();
            mesh.Compact();
            mesh.Scale(0.1);
            randomlyRotateMesh(mesh);

            Params.Input[0].VolatileData.Graft(GH_GraftMode.None);
            var point = new GH_Point();
            DA.GetData(0, ref point);
            var p = point.Value;
            mesh.Translate(new Vector3d(p));

            var colors = new List<GH_Colour>();
            DA.GetDataList(1, colors);
            foreach (var color in colors)
            {
                mesh.VertexColors.Add(color.Value);
            }

            DA.SetData(0, mesh);
        }

        protected override System.Drawing.Bitmap Icon
        {
            get
            {
                return null;
            }
        }

        public override Guid ComponentGuid
        {
            get { return new Guid("8fa5cf36-0696-441c-9041-cbe82ca9d776"); }
        }
    }
}